package org.ow2.util.deployment.annotations.analyzer;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;

import org.ow2.easybeans.asm.ClassReader;
import org.ow2.util.deployment.annotations.analyzer.configurator.ArchiveConfigurator;
import org.ow2.util.ee.deploy.api.archive.ArchiveException;
import org.ow2.util.ee.deploy.api.archive.IArchive;

/**
 * ArchiveScanner.
 * @author Gael Lalire
 */
public final class ArchiveScanner {

    /**
     * the archive.
     */
    private IArchive archive;

    /**
     * the classes in archive to scan.
     */
    private List<URL> notScannedURL;

    /**
     * a list of configurator.
     */
    private List<ArchiveConfigurator> archiveConfigurators;

    /**
     * true if endScan has been called.
     */
    private boolean scanEnded;

    /**
     * Constructor.
     * @param archive the archive to scan
     */
    public ArchiveScanner(final IArchive archive) {
        this.archive = archive;
        archiveConfigurators = new ArrayList<ArchiveConfigurator>();
        scanEnded = false;
    }

    /**
     * prepare the scan.
     * @throws AnalyzerException when resources access failed
     */
    private void prepareScan() throws AnalyzerException {
        Iterator<URL> iterator;
        try {
            iterator = archive.getResources();
        } catch (ArchiveException e) {
            throw new AnalyzerException("Error while analyzing archive '" + archive.getName() + "'", e);
        }
        notScannedURL = new ArrayList<URL>();
        while (iterator.hasNext()) {
            URL url = iterator.next();
            // only .class
            String className = url.getPath().toLowerCase();
            if (className.endsWith(".class")) {
                notScannedURL.add(url);
            }
        }
    }

    /**
     * add a configurator.
     * @param archiveConfigurator an archive configurator
     */
    public void addArchiveConfigurator(final ArchiveConfigurator archiveConfigurator) {
        archiveConfigurators.add(archiveConfigurator);
    }

    /**
     * remove a configurator.
     * @param archiveConfigurator an archive configurator
     */
    public void removeArchiveConfigurator(final ArchiveConfigurator archiveConfigurator) {
        archiveConfigurators.remove(archiveConfigurator);
    }

    /**
     * scan the class contained in url.
     * @param url the url
     * @throws AnalyzerException if error occurs
     */
    private void doScan(final URL url) throws AnalyzerException {
        InputStream is = null;
        try {
            URLConnection urlConnection = url.openConnection();
            urlConnection.setDefaultUseCaches(false);
            is = urlConnection.getInputStream();
            new ClassReader(is).accept(new ScanClassVisitor(archiveConfigurators), 0);
        } catch (Exception ioe) {
            throw new AnalyzerException("Error while analyzing file entry '" + url + "' in archive '"
                    + archive.getName() + "'", ioe);
        } finally {
            if (is != null) {
                try {
                    is.close();
                } catch (IOException e) {
                    throw new AnalyzerException("Error while closing input stream of the entry '" + url
                            + "' in archive '" + archive.getName() + "'", e);
                }
            }
        }
    }

    /**
     * scan all resources in archive and close the archive.
     * @throws AnalyzerException if error occurs
     */
    public void scanAll() throws AnalyzerException {
        scanAll(true);
    }

    /**
     * scan all resources in archive.
     * @param closeArchive close the archive
     * @throws AnalyzerException if error occurs
     */
    public void scanAll(final boolean closeArchive) throws AnalyzerException {
        if (scanEnded) {
            throw new IllegalStateException("the scan is already terminated");
        }
        if (notScannedURL == null) {
            prepareScan();
        }
        for (URL url : notScannedURL) {
            doScan(url);
        }
        endScan(closeArchive);
    }

    /**
     * scan all resources in archive contained in collection.
     * @param classNameCollection collection of class name
     * @throws AnalyzerException if error occurs
     */
    public void scanCollection(final Collection<String> classNameCollection) throws AnalyzerException {
        if (scanEnded) {
            throw new IllegalStateException("the scan is already terminated");
        }
        if (notScannedURL == null) {
            prepareScan();
        }
        for (String className : classNameCollection) {
            try {
                URL url = archive.getResource(className.replace('.', '/').concat(".class"));
                if (url != null && notScannedURL.contains(url)) {
                    doScan(url);
                    notScannedURL.remove(url);
                }
            } catch (ArchiveException e) {
                throw new AnalyzerException("Error while analyzing archive '" + archive.getName() + "'", e);
            }
        }
    }

    /**
     * End the scan and close the archive.
     */
    public void endScan() {
        endScan(true);
    }

    /**
     * End the scan.
     * @param closeArchive if true then the archive will be closed
     */
    public void endScan(final boolean closeArchive) {
        if (scanEnded) {
            throw new IllegalStateException("the scan is already terminated");
        }
        notScannedURL.clear();
        if (closeArchive) {
            archive.close();
        }
        scanEnded = true;
    }

}
