/**
 * EasyBeans
 * Copyright (C) 2006 Bull S.A.S.
 * Contact: easybeans@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: ClassMetadata.java 2234 2008-01-17 16:43:23Z gaellalire $
 * --------------------------------------------------------------------------
 */

package org.ow2.util.deployment.metadata;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.ow2.util.deployment.metadata.structures.JClass;
import org.ow2.util.deployment.metadata.structures.JField;
import org.ow2.util.deployment.metadata.structures.JMethod;


/**
 * @author Gael Lalire
 * @param <C> type of class metadata
 * @param <M> type of method metadata
 * @param <F> type of field metadata
 */
public abstract class ClassMetadata<
    C extends ClassMetadata<C, M, F>,
    M extends MethodMetadata<C, M, F>,
    F extends FieldMetadata<C, M, F>>
        extends CommonMetadata {

    /**
     * Class.
     */
    private JClass jClass;

    /**
     * List of method annotations metadata.
     */
    private Map<JMethod, M> methodsAnnotationMetadata = null;

    /**
     * List of field annotations metadata.
     */
    private Map<JField, F> fieldsAnnotationMetadata = null;


    /**
     * Constructor.
     * @param jClass asm class representation
     */
    public ClassMetadata(final JClass jClass) {
        this.jClass = jClass;
        this.methodsAnnotationMetadata = new HashMap<JMethod, M>();
        this.fieldsAnnotationMetadata = new HashMap<JField, F>();
    }


    /**
     * add a method metadata.
     * @param methodAnnotationMetadata the method metadata
     */
    public final void addMethodMetadata(final M methodAnnotationMetadata) {
        JMethod key = methodAnnotationMetadata.getJMethod();
        // No check: the same method can be added from the super class (ie : super = private, current = private)
        // So it means that there is no overriding
        methodsAnnotationMetadata.put(key, methodAnnotationMetadata);
    }

    /**
     * @param jMethod key of the map of methods annotations.
     * @return method annotation metadata of a given method.
     */
    public final M getMethodMetadata(final JMethod jMethod) {
        return methodsAnnotationMetadata.get(jMethod);
    }

    /**
     * @param methodName the name of the method to search
     * @return list of method annotation metadata for the given method name
     */
    public final List<M> searchMethodMetadata(final String methodName) {
        if (methodName == null) {
            throw new IllegalArgumentException("Cannot search null methods");
        }
        List<M> list = new ArrayList<M>();
        for (M method : methodsAnnotationMetadata.values()) {
            if (methodName.equals(method.getJMethod().getName())) {
                list.add(method);
            }
        }
        return list;
    }

    /**
     * @param fieldName the name of the field to search
     * @return list of field annotation metadata for the given field name
     */
    public final List<F> searchFieldMetadata(final String fieldName) {
        if (fieldName == null) {
            throw new IllegalArgumentException("Cannot search null fields");
        }
        List<F> list = new ArrayList<F>();
        for (F field : fieldsAnnotationMetadata.values()) {
            if (fieldName.equals(field.getJField().getName())) {
                list.add(field);
            }
        }
        return list;
    }

    /**
     * Get collections of methods annotation metadata.
     * @return collections of methods annotation metadata.
     */
    public final Collection<M> getMethodMetadataCollection() {
        return methodsAnnotationMetadata.values();
    }


    /**
     * Add field annotation metadata for a given Bean.
     * @param fieldAnnotationMetadata metadata of a field.
     */
    public final void addFieldMetadata(final F fieldAnnotationMetadata) {
        JField key = fieldAnnotationMetadata.getJField();
        // already exists ?
        if (fieldsAnnotationMetadata.containsKey(key)) {
            throw new IllegalStateException();
        }
        fieldsAnnotationMetadata.put(key, fieldAnnotationMetadata);
    }

    /**
     * @param jField key of the map of fields annotations.
     * @return field annotation metadata of a given method.
     */
    public final F getFieldMetadata(final JField jField) {
        return fieldsAnnotationMetadata.get(jField);
    }

    /**
     * Get collections of fields annotation metadata.
     * @return collections of fields annotation metadata.
     */
    public final Collection<F> getFieldMetadataCollection() {
        return fieldsAnnotationMetadata.values();
    }


    /**
     * @return class representation
     */
    public final JClass getJClass() {
        return jClass;
    }


    /**
     * @param jClass jClass
     */
    public final void setJClass(final JClass jClass) {
        this.jClass = jClass;
    }

}
